@@ -37,18 +37,26 @@
  * Interfaces for reading Ogg files and streams
  */
 
-/** \defgroup read_api OGGZ Read API
+/** \defgroup read_api Oggz Read API
  *
- * OGGZ parses Ogg bitstreams, forming ogg_packet structures, and calling
+ * Oggz parses Ogg bitstreams, forming ogg_packet structures, and calling
  * your OggzReadPacket callback(s).
  *
- * You provide Ogg data to OGGZ with oggz_read() or oggz_read_input(), and
+ * You provide Ogg data to Oggz with oggz_read() or oggz_read_input(), and
  * independently process it in OggzReadPacket callbacks.
  * It is possible to set a different callback per \a serialno (ie. for each
  * logical bitstream in the Ogg bitstream - see the \ref basics section for
  * more detail).
  *
- * See \ref seek_api for information on seeking on interleaved Ogg data.
+ * When using an OGGZ* opened with the OGGZ_AUTO flag set, Oggz will
+ * internally calculate the granulepos for each packet, even though these
+ * are not all recorded in the file: only the last packet ending on a page
+ * will have its granulepos recorded in the page header.
+ * Within a OggzReadPacket callback, calling oggz_tell_granulepos() will
+ * retrieve the calculated granulepos.
+ *
+ * See \ref seek_api for information on seeking on interleaved Ogg data,
+ * and for working with calculated granulepos values.
  *
  * \{
  */
@@ -63,7 +71,7 @@
  * \param serialno Identify the logical bistream in \a oggz that contains
  *                 \a op
  * \param user_data A generic pointer you have provided earlier
- * \returns 0 to continue, non-zero to instruct OGGZ to stop.
+ * \returns 0 to continue, non-zero to instruct Oggz to stop.
  *
  * \note It is possible to provide different callbacks per logical
  * bitstream -- see oggz_set_read_callback() for more information.
@@ -105,7 +113,7 @@ int oggz_set_read_callback (OGGZ * oggz, long serialno,
  * \param oggz The OGGZ handle
  * \param op The full ogg_page (see <ogg/ogg.h>)
  * \param user_data A generic pointer you have provided earlier
- * \returns 0 to continue, non-zero to instruct OGGZ to stop.
+ * \returns 0 to continue, non-zero to instruct Oggz to stop.
  */
 typedef int (*OggzReadPage) (OGGZ * oggz, const ogg_page * og,
 			     long serialno, void * user_data);
@@ -170,7 +178,7 @@ long oggz_read_input (OGGZ * oggz, unsigned char * buf, long n);
  */
 
 /**
- * Erase any input buffered in OGGZ. This discards any input read from the
+ * Erase any input buffered in Oggz. This discards any input read from the
  * underlying IO system but not yet delivered as ogg_packets.
  *
  * \param oggz An OGGZ handle
@@ -181,4 +189,42 @@ long oggz_read_input (OGGZ * oggz, unsigned char * buf, long n);
  */
 int oggz_purge (OGGZ * oggz);
 
+/**
+ * Determine the content type of the oggz stream referred to by \a serialno
+ *
+ * \param oggz An OGGZ handle
+ * \param serialno An ogg stream serialno
+ * \retval OGGZ_CONTENT_THEORA..OGGZ_CONTENT_UNKNOWN content successfully 
+ *          identified
+ * \retval OGGZ_ERR_BAD_OGGZ \a oggz does not refer to an existing OGGZ
+ * \retval OGGZ_ERR_BAD_SERIALNO \a serialno does not refer to an existing
+ *          stream
+ */
+OggzStreamContent oggz_stream_get_content (OGGZ * oggz, long serialno);
+
+/**
+ * Return human-readable string representation of content type of oggz stream
+ * referred to by \a serialno
+ *
+ * \param oggz An OGGZ handle
+ * \param serialno An ogg stream serialno
+ * \retval string the name of the content type
+ * \retval NULL \a oggz or \a serialno invalid
+ */
+const char * oggz_stream_get_content_type (OGGZ *oggz, long serialno);
+
+/**
+ * Determine the number of headers of the oggz stream referred to by
+ * \a serialno
+ *
+ * \param oggz An OGGZ handle
+ * \param serialno An ogg stream serialno
+ * \retval OGGZ_CONTENT_THEORA..OGGZ_CONTENT_UNKNOWN content successfully 
+ *          identified
+ * \retval OGGZ_ERR_BAD_OGGZ \a oggz does not refer to an existing OGGZ
+ * \retval OGGZ_ERR_BAD_SERIALNO \a serialno does not refer to an existing
+ *          stream
+ */
+int oggz_stream_get_numheaders (OGGZ * oggz, long serialno);
+
 #endif /* __OGGZ_READ_H__ */
