@@ -219,7 +219,7 @@ This was the Theora header for theora-alpha2:
 */
 
 /**
- * fishead (Annodex version 3.0)
+ * Skeleton BOS page ("fishead", Annodex version 3.0)
  *
  * Default field type: LITTLE ENDIAN unsigned integer
 
@@ -261,7 +261,7 @@ This was the Theora header for theora-alpha2:
 */
 
 /**
- * fisbone (Annodex version 3.0)
+ * Skeleton track info ("fisbone", Annodex version 3.0)
  *
  * Default field type: LITTLE ENDIAN unsigned integer
 
@@ -428,6 +428,50 @@ This was the Theora header for theora-alpha2:
 
  */
 
+/**
+ * Kate bitstream version 0.1
+ *
+ * Default field type: LITTLE ENDIAN unsigned integer
+
+ 0               1               2               3              |
+ 0                   1                   2                   3
+ 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1| Byte
++-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+| packtype      | Identifier char[7]: 'kate\0\0\0'              | 0-3
++-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+| kate magic continued                                          | 4-7
++-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+| reserved - 0  | version major | version minor | num headers   | 8-11
++-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+| text encoding | directionality| reserved - 0  | granule shift | 12-15
++-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+| reserved - 0                                                  | 16-19
++-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+| reserved - 0                                                  | 20-23
++-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+| granule rate numerator                                        | 24-27
++-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+| granule rate denominator                                      | 28-31
++-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+| language (NUL terminated)                                     | 32-35
++-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+| language (continued)                                          | 36-39
++-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+| language (continued)                                          | 40-43
++-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+| language (continued)                                          | 44-47
++-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+| category (NUL terminated)                                     | 48-51
++-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+| category (continued)                                          | 52-55
++-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+| category (continued)                                          | 56-59
++-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+| category (continued)                                          | 60-63
++-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+
+ */
+
 int oggz_auto_identify (OGGZ *oggz, ogg_page *og, long serialno);
 
 #endif /* __OGGZ_AUTO_H__ */
