@@ -733,6 +733,8 @@ oggz_comment_generate(OGGZ * oggz, long serialno,
     {0x81, 0x74, 0x68, 0x65, 0x6f, 0x72, 0x61};
   const unsigned char preamble_flac[4] =
     {0x04, 0x00, 0x00, 0x00};
+  const unsigned char preamble_kate[9] =
+    {0x81, 0x6b, 0x61, 0x74, 0x65, 0x00, 0x00, 0x00, 0x00};
 
 
   switch(packet_type) {
@@ -748,6 +750,10 @@ oggz_comment_generate(OGGZ * oggz, long serialno,
       preamble_length = sizeof preamble_flac;
       preamble = preamble_flac;
       break;
+    case OGGZ_CONTENT_KATE:
+      preamble_length = sizeof preamble_kate;
+      preamble = preamble_kate;
+      break;
     case OGGZ_CONTENT_PCM:
     case OGGZ_CONTENT_SPEEX:
       preamble_length = 0;
@@ -808,7 +814,7 @@ oggz_comment_generate(OGGZ * oggz, long serialno,
   return c_packet;
 }
 
-/* In Flac, OggPCM, Speex, Theora and Vorbis the comment packet will
+/* In Flac, OggPCM, Speex, Theora Vorbis, and Kate the comment packet will
    be second in the stream, i.e. packetno=1, and it will have granulepos=0 */
 ogg_packet *
 oggz_comments_generate(OGGZ * oggz, long serialno,
