@@ -38,23 +38,27 @@
 
 #include <ogg/ogg.h>
 #include <oggz/oggz_constants.h>
+#include <oggz/oggz_off_t.h>
 
 #include "oggz_macros.h"
 #include "oggz_vector.h"
+#include "oggz_dlist.h"
 
 typedef struct _OGGZ OGGZ;
+typedef struct _OggzComment OggzComment;
 typedef struct _OggzIO OggzIO;
 typedef struct _OggzReader OggzReader;
 typedef struct _OggzWriter OggzWriter;
 
-/* oggz_stream */
-#include "oggz_stream.h"
 
 typedef int (*OggzReadPacket) (OGGZ * oggz, ogg_packet * op, long serialno,
 			       void * user_data);
 typedef int (*OggzReadPage) (OGGZ * oggz, const ogg_page * og, long serialno,
 			     void * user_data);
 
+/* oggz_stream */
+#include "oggz_stream_private.h"
+
 typedef ogg_int64_t (*OggzMetric) (OGGZ * oggz, long serialno,
 				   ogg_int64_t granulepos,
 				   void * user_data);
@@ -76,13 +80,17 @@ struct _oggz_stream_t {
 
   /** STATIC INFO */
   int content;
-  int nr_headers;
+  int numheaders;
   int preroll;
   ogg_int64_t granulerate_n;
   ogg_int64_t granulerate_d;
   ogg_int64_t basegranule;
   int granuleshift;
 
+  /* The comments */
+  char * vendor;
+  OggzVector * comments;
+
   /** CURRENT STATE **/
   /* non b_o_s packet has been written (not just queued) */
   int delivered_non_b_o_s;
@@ -105,6 +113,12 @@ struct _oggz_stream_t {
 
   OggzReadPage read_page;
   void * read_page_user_data;
+
+  /* calculated granulepos values, not extracted values */
+  ogg_int64_t last_granulepos;
+  ogg_int64_t page_granulepos;
+  void * calculate_data;
+  ogg_packet  * last_packet;
 };
 
 struct _OggzReader {
@@ -121,9 +135,10 @@ struct _OggzReader {
   void * read_page_user_data;
 
   ogg_int64_t current_unit;
+  ogg_int64_t current_granulepos;
 
 #if 0
-  off_t offset_page_end; /* offset of end of current page */
+  oggz_off_t offset_page_end; /* offset of end of current page */
 #endif
 };
 
@@ -168,6 +183,10 @@ struct _OggzWriter {
   int page_offset; /* n bytes already copied out of current page */
 
   ogg_stream_state * current_stream;
+
+  int no_more_packets; /* used only in the local oggz_write loop to indicate
+                          end of stream */
+
 };
 
 struct _OggzIO {
@@ -187,6 +206,14 @@ struct _OggzIO {
   void * flush_user_handle;
 };
 
+struct _OggzComment {
+  /** The name of the comment, eg. "AUTHOR" */
+  char * name;
+
+  /** The value of the comment, as UTF-8 */
+  char * value;
+};
+
 struct _OGGZ {
   int flags;
   FILE * file;
@@ -195,8 +222,8 @@ struct _OGGZ {
   ogg_packet current_packet;
   ogg_page current_page;
 
-  off_t offset; /* offset of current page start */
-  off_t offset_data_begin; /* offset of unit 0 page start */
+  oggz_off_t offset; /* offset of current page start */
+  oggz_off_t offset_data_begin; /* offset of unit 0 page start */
 
   long run_blocksize; /* blocksize to use for oggz_run() */
   int cb_next;
@@ -215,6 +242,8 @@ struct _OGGZ {
     OggzReader reader;
     OggzWriter writer;
   } x;
+
+  OggzDList * packet_buffer;
 };
 
 OGGZ * oggz_read_init (OGGZ * oggz);
@@ -235,7 +264,42 @@ int oggz_has_metrics (OGGZ * oggz);
 
 int oggz_purge (OGGZ * oggz);
 
-int oggz_auto (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data);
+/* metric_internal */
+
+int
+oggz_set_granulerate (OGGZ * oggz, long serialno, 
+                                    ogg_int64_t granule_rate_numerator,
+                                    ogg_int64_t granule_rate_denominator);
+
+int
+oggz_get_granulerate (OGGZ * oggz, long serialno,
+                                    ogg_int64_t * granulerate_n,
+                                    ogg_int64_t * granulerate_d);
+
+int oggz_set_granuleshift (OGGZ * oggz, long serialno, int granuleshift);
+
+int oggz_get_granuleshift (OGGZ * oggz, long serialno);
+
+/* oggz_auto */
+ 
+int
+oggz_auto_get_granulerate (OGGZ * oggz, ogg_packet * op, long serialno, 
+                           void * user_data);
+
+int
+oggz_auto_read_comments (OGGZ * oggz, oggz_stream_t * stream, long serialno,
+                         ogg_packet * op);
+
+int oggz_auto_identify_page (OGGZ *oggz, ogg_page *og, long serialno);
+int oggz_auto_identify_packet (OGGZ * oggz, ogg_packet * op, long serialno);
+
+/* comments */
+int oggz_comments_init (oggz_stream_t * stream);
+int oggz_comments_free (oggz_stream_t * stream);
+int oggz_comments_decode (OGGZ * oggz, long serialno,
+                          unsigned char * comments, long length);
+long oggz_comments_encode (OGGZ * oggz, long serialno,
+                           unsigned char * buf, long length);
 
 /* oggz_io */
 size_t oggz_io_read (OGGZ * oggz, void * buf, size_t n);
