@@ -60,8 +60,10 @@
 #include "oggz_compat.h"
 #include "oggz_private.h"
 
-/*#define DEBUG*/
-/*#define DEBUG_VERBOSE*/
+#include <oggz/oggz_stream.h>
+
+/* #define DEBUG */
+/* #define DEBUG_VERBOSE */
 
 #define CHUNKSIZE 65536
 
@@ -100,7 +102,7 @@ oggz_read_close (OGGZ * oggz)
 
 int
 oggz_set_read_callback (OGGZ * oggz, long serialno,
-			OggzReadPacket read_packet, void * user_data)
+                        OggzReadPacket read_packet, void * user_data)
 {
   OggzReader * reader;
   oggz_stream_t * stream;
@@ -134,7 +136,7 @@ oggz_set_read_callback (OGGZ * oggz, long serialno,
 
 int
 oggz_set_read_page (OGGZ * oggz, long serialno, OggzReadPage read_page,
-		    void * user_data)
+                    void * user_data)
 {
   OggzReader * reader;
   oggz_stream_t * stream;
@@ -196,15 +198,15 @@ oggz_get_next_page_7 (OGGZ * oggz, ogg_page * og)
       buffer = ogg_sync_buffer (&reader->ogg_sync, CHUNKSIZE);
       if ((bytes = oggz_io_read (oggz, buffer, CHUNKSIZE)) == 0) {
 #if 0
-	if (ferror (oggz->file)) {
-	  oggz_set_error (oggz, OGGZ_ERR_SYSTEM);
-	  return -1;
-	}
+  if (ferror (oggz->file)) {
+    oggz_set_error (oggz, OGGZ_ERR_SYSTEM);
+    return -1;
+  }
 #endif
       }
 
       if (bytes == 0) {
-	return -2;
+        return -2;
       }
 
       ogg_sync_wrote(&reader->ogg_sync, bytes);
@@ -238,6 +240,96 @@ oggz_get_next_page_7 (OGGZ * oggz, ogg_page * og)
   return ret;
 }
 
+typedef struct {
+  ogg_packet      packet;
+  ogg_int64_t     calced_granulepos;
+  oggz_stream_t * stream;
+  OggzReader    * reader;
+  OGGZ          * oggz;
+  long            serialno;
+} OggzBufferedPacket;
+
+OggzBufferedPacket *
+oggz_read_new_pbuffer_entry(OGGZ *oggz, ogg_packet *packet, 
+            ogg_int64_t granulepos, long serialno, oggz_stream_t * stream, 
+            OggzReader *reader) {
+
+  OggzBufferedPacket *p = malloc(sizeof(OggzBufferedPacket));
+  memcpy(&(p->packet), packet, sizeof(ogg_packet));
+  p->packet.packet = malloc(packet->bytes);
+  memcpy(p->packet.packet, packet->packet, packet->bytes);
+
+  p->calced_granulepos = granulepos;
+  p->stream = stream;
+  p->serialno = serialno;
+  p->reader = reader;
+  p->oggz = oggz;
+
+  return p;
+}
+
+void
+oggz_read_free_pbuffer_entry(OggzBufferedPacket *p) {
+  
+  free(p->packet.packet);
+  free(p);
+
+}
+
+OggzDListIterResponse
+oggz_read_update_gp(void *elem) {
+
+  OggzBufferedPacket *p = (OggzBufferedPacket *)elem;
+
+  if (p->calced_granulepos == -1 && p->stream->last_granulepos != -1) {
+    int content = oggz_stream_get_content(p->oggz, p->serialno);
+    p->calced_granulepos = 
+      oggz_auto_calculate_gp_backwards(content, p->stream->last_granulepos,
+      p->stream, &(p->packet), p->stream->last_packet);
+      
+    p->stream->last_granulepos = p->calced_granulepos;
+    p->stream->last_packet = &(p->packet);
+  }
+
+  return DLIST_ITER_CONTINUE;
+
+}
+
+OggzDListIterResponse
+oggz_read_deliver_packet(void *elem) {
+
+  OggzBufferedPacket *p = (OggzBufferedPacket *)elem;
+  ogg_int64_t gp_stored;
+  ogg_int64_t unit_stored;
+
+  if (p->calced_granulepos == -1) {
+    return DLIST_ITER_CANCEL;
+  }
+
+  gp_stored = p->reader->current_granulepos;
+  unit_stored = p->reader->current_unit;
+
+  p->reader->current_granulepos = p->calced_granulepos;
+
+  p->reader->current_unit =
+    oggz_get_unit (p->oggz, p->serialno, p->calced_granulepos);
+
+  if (p->stream->read_packet) {
+    p->stream->read_packet(p->oggz, &(p->packet), p->serialno, 
+            p->stream->read_user_data);
+  } else if (p->reader->read_packet) {
+    p->reader->read_packet(p->oggz, &(p->packet), p->serialno, 
+            p->reader->read_user_data);
+  }
+
+  p->reader->current_granulepos = gp_stored;
+  p->reader->current_unit = unit_stored;
+
+  oggz_read_free_pbuffer_entry(p);
+
+  return DLIST_ITER_CONTINUE;
+}
+
 static int
 oggz_read_sync (OGGZ * oggz)
 {
@@ -258,61 +350,158 @@ oggz_read_sync (OGGZ * oggz)
 
   /* handle one packet.  Try to fetch it from current stream state */
   /* extract packets from page */
-  while(cb_ret == 0){
+  while(cb_ret == 0) {
 
     if (reader->current_serialno != -1) {
     /* process a packet if we can.  If the machine isn't loaded,
        neither is a page */
-    while(cb_ret == 0) {
-      ogg_int64_t granulepos;
-      int result;
+      while(cb_ret == 0) {
+        ogg_int64_t granulepos;
+        int result;
 
-      serialno = reader->current_serialno;
+        serialno = reader->current_serialno;
 
-      stream = oggz_get_stream (oggz, serialno);
+        stream = oggz_get_stream (oggz, serialno);
 
-      if (stream == NULL) {
-	/* new stream ... check bos etc. */
-	if ((stream = oggz_add_stream (oggz, serialno)) == NULL) {
-	  /* error -- could not add stream */
-	  return -7;
-	}
-      }
-      os = &stream->ogg_stream;
+        if (stream == NULL) {
+          /* new stream ... check bos etc. */
+          if ((stream = oggz_add_stream (oggz, serialno)) == NULL) {
+            /* error -- could not add stream */
+            return -7;
+          }
+        }
+        os = &stream->ogg_stream;
 
-      result = ogg_stream_packetout(os, op);
+        result = ogg_stream_packetout(os, op);
 
-      if(result == -1) {
+        if(result == -1) {
 #ifdef DEBUG
-	printf ("oggz_read_sync: hole in the data\n");
+          printf ("oggz_read_sync: hole in the data\n");
 #endif
-	return -7;
+          result = ogg_stream_packetout(os, op);
+          if (result == -1) {
+#ifdef DEBUG
+            /*
+             * libogg flags "holes in the data" (which are really 
+             * inconsistencies in the page sequence number) by returning
+             * -1.  This occurs in some files and pretty much doesn't matter,
+             *  so we silently swallow the notification and reget the packet.
+             *  If the result is *still* -1 then something strange is happening.
+             */
+            printf ("shouldn't get here");
+#endif
+            return -7;
+          }
+        }
+
+        if(result > 0){
+          int content;
+
+          stream->packetno++;
+          
+          /* got a packet.  process it */
+          granulepos = op->granulepos;
+
+          content = oggz_stream_get_content(oggz, serialno);
+  
+          /*
+           * if we have no metrics for this stream yet, then generate them
+           */      
+          if 
+          (
+            (!stream->metric || (content == OGGZ_CONTENT_SKELETON)) 
+            && 
+            (oggz->flags & OGGZ_AUTO)
+          ) 
+          {
+            oggz_auto_get_granulerate (oggz, op, serialno, NULL);
+          }
+
+          /* attempt to determine granulepos for this packet */
+          if (oggz->flags & OGGZ_AUTO) {
+            reader->current_granulepos = 
+              oggz_auto_calculate_granulepos (content, granulepos, stream, op); 
+            /* make sure that we accept any "real" gaps in the granulepos
+             */
+            if (granulepos != -1 && reader->current_granulepos < granulepos) {
+              reader->current_granulepos = granulepos;
+            }
+          } else {
+            reader->current_granulepos = granulepos;
+          }
+          stream->last_granulepos = reader->current_granulepos;
+        
+          /* set unit on last packet of page */
+          if 
+          (
+            (oggz->metric || stream->metric) && reader->current_granulepos != -1
+          ) 
+          {
+            reader->current_unit =
+              oggz_get_unit (oggz, serialno, reader->current_granulepos);
+          }
+
+          if (stream->packetno == 1) {
+            oggz_auto_read_comments (oggz, stream, serialno, op);
+          }
+          
+          if (oggz->flags & OGGZ_AUTO) {
+          
+            /*
+             * while we are getting invalid granulepos values, store the 
+             * incoming packets in a dlist */
+            if (reader->current_granulepos == -1) {
+              OggzBufferedPacket *p = oggz_read_new_pbuffer_entry(
+                                oggz, &packet, reader->current_granulepos, 
+                                serialno, stream, reader);
+
+              oggz_dlist_append(oggz->packet_buffer, p);
+              continue;
+            } else if (!oggz_dlist_is_empty(oggz->packet_buffer)) {
+              /*
+               * move backward through the list assigning gp values based upon
+               * the granulepos we just recieved.  Then move forward through
+               * the list delivering any packets at the beginning with valid
+               * gp values
+               */
+              ogg_int64_t gp_stored = stream->last_granulepos;
+              stream->last_packet = &packet;
+              oggz_dlist_reverse_iter(oggz->packet_buffer, oggz_read_update_gp);
+              oggz_dlist_deliter(oggz->packet_buffer, oggz_read_deliver_packet);
+
+              /*
+               * fix up the stream granulepos 
+               */
+              stream->last_granulepos = gp_stored;
+
+              if (!oggz_dlist_is_empty(oggz->packet_buffer)) {
+                OggzBufferedPacket *p = oggz_read_new_pbuffer_entry(
+                                oggz, &packet, reader->current_granulepos, 
+                                serialno, stream, reader);
+
+                oggz_dlist_append(oggz->packet_buffer, p);
+                continue;
+              }
+            }
+          }
+
+          if (stream->read_packet) {
+            cb_ret =
+              stream->read_packet (oggz, op, serialno, stream->read_user_data);
+          } else if (reader->read_packet) {
+            cb_ret =
+              reader->read_packet (oggz, op, serialno, reader->read_user_data);
+          }
+
+          /* Mark this stream as having delivered a non b_o_s packet if so.
+           * In the case where there is no packet reading callback, this is
+           * also valid as the page reading callback has already been called.
+           */
+          if (!op->b_o_s) stream->delivered_non_b_o_s = 1;
+        }
+        else
+          break;
       }
-
-      if(result > 0){
-	/* got a packet.  process it */
-	granulepos = op->granulepos;
-
-	if (!stream->metric && (oggz->flags & OGGZ_AUTO)) {
-	  oggz_auto (oggz, op, serialno, NULL);
-	}
-
-	/* set unit on last packet of page */
-	if ((oggz->metric || stream->metric) && granulepos != -1) {
-	  reader->current_unit = oggz_get_unit (oggz, serialno, granulepos);
-	}
-
-	if (stream->read_packet) {
-	  cb_ret =
-	    stream->read_packet (oggz, op, serialno, stream->read_user_data);
-	} else if (reader->read_packet) {
-	  cb_ret =
-	    reader->read_packet (oggz, op, serialno, reader->read_user_data);
-	}
-      }
-      else
-	break;
-    }
     }
 
     /* If we've got a stop already, don't read more data in */
@@ -329,30 +518,42 @@ oggz_read_sync (OGGZ * oggz)
     if (stream == NULL) {
       /* new stream ... check bos etc. */
       if ((stream = oggz_add_stream (oggz, serialno)) == NULL) {
-	/* error -- could not add stream */
-	return -7;
+        /* error -- could not add stream */
+        return -7;
       }
+
+      /* identify stream type */
+      oggz_auto_identify_page (oggz, &og, serialno);
     }
+    else if (oggz_stream_get_content(oggz, serialno) == OGGZ_CONTENT_ANXDATA)
+    {
+      /*
+       * re-identify ANXDATA streams as these are now content streams
+       */
+      oggz_auto_identify_page (oggz, &og, serialno);
+    }
+    
     os = &stream->ogg_stream;
 
     {
       ogg_int64_t granulepos;
 
       granulepos = ogg_page_granulepos (&og);
+      stream->page_granulepos = granulepos;
 
       if ((oggz->metric || stream->metric) && granulepos != -1) {
-	reader->current_unit = oggz_get_unit (oggz, serialno, granulepos);
+       reader->current_unit = oggz_get_unit (oggz, serialno, granulepos);
       } else if (granulepos == 0) {
-	reader->current_unit = 0;
+       reader->current_unit = 0;
       }
     }
 
     if (stream->read_page) {
       cb_ret =
-	stream->read_page (oggz, &og, serialno, stream->read_page_user_data);
+        stream->read_page (oggz, &og, serialno, stream->read_page_user_data);
     } else if (reader->read_page) {
-      cb_ret = reader->read_page (oggz, &og, serialno,
-				  reader->read_page_user_data);
+      cb_ret =
+        reader->read_page (oggz, &og, serialno, reader->read_page_user_data);
     }
 
 #if 0
@@ -397,7 +598,7 @@ oggz_read (OGGZ * oggz, long n)
     else {
 #if 0
       printf ("oggz_read: EMPTY, current_unit %ld != 0\n",
-	      reader->current_unit);
+              reader->current_unit);
       return 0;
 #endif
     }
@@ -405,7 +606,7 @@ oggz_read (OGGZ * oggz, long n)
 #endif
 
   while (cb_ret != OGGZ_STOP_ERR && cb_ret != OGGZ_STOP_OK &&
-	 bytes_read > 0 && remaining > 0) {
+         bytes_read > 0 && remaining > 0) {
     bytes = MIN (remaining, CHUNKSIZE);
     buffer = ogg_sync_buffer (&reader->ogg_sync, bytes);
     if ((bytes_read = (long) oggz_io_read (oggz, buffer, bytes)) == 0) {
@@ -483,7 +684,7 @@ oggz_read_input (OGGZ * oggz, unsigned char * buf, long n)
 #endif
 
   while (cb_ret != OGGZ_STOP_ERR && cb_ret != OGGZ_STOP_OK  &&
-	 /* !oggz->eos && */ remaining > 0) {
+         /* !oggz->eos && */ remaining > 0) {
     bytes = MIN (remaining, 4096);
     buffer = ogg_sync_buffer (&reader->ogg_sync, bytes);
     memcpy (buffer, buf, bytes);
@@ -533,7 +734,7 @@ oggz_read_close (OGGZ * oggz)
 
 int
 oggz_set_read_callback (OGGZ * oggz, long serialno,
-			OggzReadPacket read_packet, void * user_data)
+                        OggzReadPacket read_packet, void * user_data)
 {
   return OGGZ_ERR_DISABLED;
 }
