@@ -95,11 +95,18 @@ oggz_tell_raw (OGGZ * oggz)
 /*
  * seeks and syncs
  */
+
+int
+oggz_seek_reset_stream(void *data) {
+  ((oggz_stream_t *)data)->last_granulepos = -1L;
+  return 0;
+}
+
 static oggz_off_t
 oggz_seek_raw (OGGZ * oggz, oggz_off_t offset, int whence)
 {
-  OggzReader * reader = &oggz->x.reader;
-  oggz_off_t offset_at;
+  OggzReader  * reader = &oggz->x.reader;
+  oggz_off_t    offset_at;
 
   if (oggz_io_seek (oggz, offset, whence) == -1) {
     return -1;
@@ -111,6 +118,8 @@ oggz_seek_raw (OGGZ * oggz, oggz_off_t offset, int whence)
 
   ogg_sync_reset (&reader->ogg_sync);
 
+  oggz_vector_foreach(oggz->streams, oggz_seek_reset_stream);
+  
   return offset_at;
 }
 
@@ -145,7 +154,7 @@ oggz_reset_seek (OGGZ * oggz, oggz_off_t offset, ogg_int64_t unit, int whence)
   oggz->offset = offset_at;
 
 #ifdef DEBUG
-  printf ("reset to %ld\n", offset_at);
+  printf ("reset to %" PRI_OGGZ_OFF_T "d\n", offset_at);
 #endif
 
   if (unit != -1) reader->current_unit = unit;
@@ -276,8 +285,9 @@ oggz_get_next_start_page (OGGZ * oggz, ogg_page * og)
      *   !ogg_page_continued : page contains start of a packet
      *   ogg_page_packets > 1: page contains start of a packet
      */
-    if (page_offset <= 0 || !ogg_page_continued (og) ||
-	ogg_page_packets (og) > 1)
+    /*if (page_offset <= 0 || !ogg_page_continued (og) ||
+	ogg_page_packets (og) > 1)*/
+    if (page_offset <= 0 || ogg_page_granulepos(og) > -1)
       found = 1;
   }
   while (!found);
@@ -313,10 +323,10 @@ oggz_get_prev_start_page (OGGZ * oggz, ogg_page * og,
 
 #ifdef DEBUG
 
-    printf ("get_prev_start_page: [A] offset_at: @%ld\toffset_start: @%ld\n",
+    printf ("get_prev_start_page: [A] offset_at: @%" PRI_OGGZ_OFF_T "d\toffset_start: @%" PRI_OGGZ_OFF_T "d\n",
 	    offset_at, offset_start);
 
-    printf ("get_prev_start_page: seeked to %ld\n", offset_start);
+    printf ("get_prev_start_page: seeked to %" PRI_OGGZ_OFF_T "d\n", offset_start);
 #endif
 
     page_offset = 0;
@@ -339,7 +349,7 @@ oggz_get_prev_start_page (OGGZ * oggz, ogg_page * og,
       granule_at = ogg_page_granulepos (og);
 
 #ifdef DEBUG_VERBOSE
-      printf ("get_prev_start_page: GOT page (%lld) @%ld\tat @%ld\n",
+      printf ("get_prev_start_page: GOT page (%lld) @%" PRI_OGGZ_OFF_T "d\tat @%" PRI_OGGZ_OFF_T  "d\n",
 	      granule_at, page_offset, offset_at);
 #endif
 
@@ -355,8 +365,8 @@ oggz_get_prev_start_page (OGGZ * oggz, ogg_page * og,
     } while (page_offset >= 0 && page_offset < offset_at);
 
 #ifdef DEBUG
-    printf ("get_prev_start_page: [B] offset_at: @%ld\toffset_start: @%ld\n"
-	    "found_offset: @%ld\tpage_offset: @%ld\n",
+    printf ("get_prev_start_page: [B] offset_at: @%" PRI_OGGZ_OFF_T "d\toffset_start: @%" PRI_OGGZ_OFF_T "d\n"
+	    "found_offset: @%" PRI_OGGZ_OFF_T "d\tpage_offset: @%" PRI_OGGZ_OFF_T "d\n",
 	    offset_at, offset_start, found_offset, page_offset);
 #endif
     /* reset the file offset */
@@ -368,8 +378,8 @@ oggz_get_prev_start_page (OGGZ * oggz, ogg_page * og,
   offset_at = oggz_reset (oggz, found_offset, unit_at, SEEK_SET);
 
 #ifdef DEBUG
-    printf ("get_prev_start_page: [C] offset_at: @%ld\t"
-	    "found_offset: @%ld\tunit_at: %lld\n",
+    printf ("get_prev_start_page: [C] offset_at: @%" PRI_OGGZ_OFF_T "d\t"
+	    "found_offset: @%" PRI_OGGZ_OFF_T "d\tunit_at: %lld\n",
 	    offset_at, found_offset, unit_at);
 #endif
 
@@ -392,7 +402,7 @@ oggz_scan_for_page (OGGZ * oggz, ogg_page * og, ogg_int64_t unit_target,
   long serialno;
 
 #ifdef DEBUG
-  printf (" SCANNING from %ld...", offset_begin);
+  printf (" SCANNING from %" PRI_OGGZ_OFF_T "d...", offset_begin);
 #endif
 
   for ( ; ; ) {
@@ -400,7 +410,7 @@ oggz_scan_for_page (OGGZ * oggz, ogg_page * og, ogg_int64_t unit_target,
     if (offset_at == -1) return -1;
 
 #ifdef DEBUG
-    printf (" scan @%ld\n", offset_at);
+    printf (" scan @%" PRI_OGGZ_OFF_T "d\n", offset_at);
 #endif
 
     offset_next = oggz_get_next_start_page (oggz, og);
@@ -475,6 +485,10 @@ oggz_scan_for_page (OGGZ * oggz, ogg_page * og, ogg_int64_t unit_target,
 
       break;
 #else
+      do {
+        offset_at = oggz_get_prev_start_page(oggz, og, &granule_at, &serialno);
+        unit_at = oggz_get_unit(oggz, serialno, granule_at);
+      } while (unit_at > unit_target);
       return offset_at;
 #endif
     } else if (unit_at == unit_target) {
@@ -540,7 +554,7 @@ oggz_seek_guess (ogg_int64_t unit_at, ogg_int64_t unit_target,
   }
 
 #ifdef DEBUG
-    printf ("oggz_seek_guess: guessed %ld\n", offset_guess);
+    printf ("oggz_seek_guess: guessed %" PRI_OGGZ_OFF_T "d\n", offset_guess);
 #endif
 
   return offset_guess;
@@ -567,7 +581,7 @@ oggz_offset_end (OGGZ * oggz)
     if (oggz_stat_regular (statbuf.st_mode)) {
       offset_end = statbuf.st_size;
 #ifdef DEBUG
-      printf ("oggz_offset_end: stat size %ld\n", offset_end);
+      printf ("oggz_offset_end: stat size %" PRI_OGGZ_OFF_T "d\n", offset_end);
 #endif
     } else {
       /*oggz_set_error (oggz, OGGZ_ERR_NOSEEK);*/
@@ -659,7 +673,7 @@ oggz_seek_set (OGGZ * oggz, ogg_int64_t unit_target)
     hit_eof = 0;
 
 #ifdef DEBUG
-    printf ("oggz_seek_set: [A] want u%lld: (u%lld - u%lld) [@%ld - @%ld]\n",
+    printf ("oggz_seek_set: [A] want u%lld: (u%lld - u%lld) [@%" PRI_OGGZ_OFF_T "d - @%" PRI_OGGZ_OFF_T "d]\n",
 	    unit_target, unit_begin, unit_end, offset_begin, offset_end);
 #endif
 
@@ -682,7 +696,7 @@ oggz_seek_set (OGGZ * oggz, ogg_int64_t unit_target)
     offset_next = oggz_get_next_start_page (oggz, og);
 
 #ifdef DEBUG
-    printf ("oggz_seek_set: offset_next %ld\n", offset_next);
+    printf ("oggz_seek_set: offset_next %" PRI_OGGZ_OFF_T "d\n", offset_next);
 #endif
 
     if (/*unit_end == -1 &&*/ offset_next == -2) { /* reached eof, backtrack */
@@ -691,7 +705,7 @@ oggz_seek_set (OGGZ * oggz, ogg_int64_t unit_target)
 					      &serialno);
       unit_end = oggz_get_unit (oggz, serialno, granule_at);
 #ifdef DEBUG
-      printf ("oggz_seek_set: [C] offset_next @%ld, g%lld, (s%ld)\n",
+      printf ("oggz_seek_set: [C] offset_next @%" PRI_OGGZ_OFF_T "d, g%lld, (s%ld)\n",
 	      offset_next, granule_at, serialno);
       printf ("oggz_seek_set: [c] u%lld\n",
 	      oggz_get_unit (oggz, serialno, granule_at));
@@ -726,7 +740,7 @@ oggz_seek_set (OGGZ * oggz, ogg_int64_t unit_target)
     if (unit_at == unit_last_iter) break;
 
 #ifdef DEBUG
-    printf ("oggz_seek_set: [D] want u%lld, got page u%lld @%ld g%lld\n",
+    printf ("oggz_seek_set: [D] want u%lld, got page u%lld @%" PRI_OGGZ_OFF_T "d g%lld\n",
 	    unit_target, unit_at, offset_at, granule_at);
 #endif
 
@@ -744,9 +758,10 @@ oggz_seek_set (OGGZ * oggz, ogg_int64_t unit_target)
   }
 
  found:
-  offset_at = oggz_get_prev_start_page (oggz, og, &granule_at, &serialno);
-
-  unit_at = oggz_get_unit (oggz, serialno, granule_at);
+  do {
+    offset_at = oggz_get_prev_start_page (oggz, og, &granule_at, &serialno);
+    unit_at = oggz_get_unit (oggz, serialno, granule_at);
+  } while (unit_at > unit_target);
 
   if (offset_at < 0) {
     oggz_reset (oggz, offset_orig, -1, SEEK_SET);
@@ -798,7 +813,7 @@ oggz_seek_end (OGGZ * oggz, ogg_int64_t unit_offset)
   }
 
 #ifdef DEBUG
-  printf ("*** oggz_seek_end: found packet (%lld) at @%ld [%lld]\n",
+  printf ("*** oggz_seek_end: found packet (%lld) at @%" PRI_OGGZ_OFF_T "d [%lld]\n",
 	  unit_end, offset_end, granulepos);
 #endif
 
@@ -832,6 +847,8 @@ oggz_seek_units (OGGZ * oggz, ogg_int64_t units, int whence)
 {
   OggzReader * reader = &oggz->x.reader;
 
+  ogg_int64_t r;
+
   if (oggz == NULL) {
 #ifdef DEBUG
     printf ("oggz_seek_units: oggz NULL, FAIL\n");
@@ -855,20 +872,23 @@ oggz_seek_units (OGGZ * oggz, ogg_int64_t units, int whence)
 
   switch (whence) {
   case SEEK_SET:
-    return oggz_seek_set (oggz, units);
+    r = oggz_seek_set (oggz, units);
     break;
   case SEEK_CUR:
     units += reader->current_unit;
-    return oggz_seek_set (oggz, units);
+    r = oggz_seek_set (oggz, units);
     break;
   case SEEK_END:
-    return oggz_seek_end (oggz, units);
+    r = oggz_seek_end (oggz, units);
     break;
   default:
     /*oggz_set_error (oggz, OGGZ_EINVALID);*/
-    return -1;
+    r = -1;
     break;
   }
+
+  reader->current_granulepos = -1;
+  return r;
 }
 
 long
