@@ -103,6 +103,7 @@ oggz_write_init (OGGZ * oggz)
   writer->hungry_only_when_empty = 0;
 
   writer->writing = 0;
+  writer->no_more_packets = 0;
   writer->state = OGGZ_MAKING_PACKETS;
 
   writer->flushing = 0;
@@ -163,6 +164,9 @@ oggz_write_close (OGGZ * oggz)
 
   oggz_write_flush (oggz);
 
+  oggz_writer_packet_free (writer->current_zpacket);
+  oggz_writer_packet_free (writer->next_zpacket);
+
   oggz_vector_foreach (writer->packet_queue,
 		       (OggzFunc)oggz_writer_packet_free);
   oggz_vector_delete (writer->packet_queue);
@@ -219,6 +223,15 @@ oggz_write_feed (OGGZ * oggz, ogg_packet * op, long serialno, int flush,
 
   if (guard && *guard != 0) return OGGZ_ERR_BAD_GUARD;
 
+  /* Check that the serialno is in the valid range for an Ogg page header,
+   * ie. that it fits within 32 bits and does not equal the special value -1 */
+  if ((long)((ogg_int32_t)serialno) != serialno || serialno == -1) {
+#ifdef DEBUG
+    printf ("oggz_write_feed: serialno %010ld\n", serialno);
+#endif
+    return OGGZ_ERR_BAD_SERIALNO;
+  }
+
 #ifdef DEBUG
   printf ("oggz_write_feed: (%010ld) FLUSH: %d\n", serialno, flush);
 #endif
@@ -243,6 +256,7 @@ oggz_write_feed (OGGZ * oggz, ogg_packet * op, long serialno, int flush,
 
     if (b_o_s || !strict || suffix) {
       stream = oggz_add_stream (oggz, serialno);
+      oggz_auto_identify_packet (oggz, op, serialno);
     } else {
       return OGGZ_ERR_BAD_SERIALNO;
     }
@@ -257,7 +271,6 @@ oggz_write_feed (OGGZ * oggz, ogg_packet * op, long serialno, int flush,
   if (strict) {
     if (op->bytes < 0) return OGGZ_ERR_BAD_BYTES;
     if (!suffix && b_o_s != stream->b_o_s) return OGGZ_ERR_BAD_B_O_S;
-
     if (op->granulepos != -1 && op->granulepos < stream->granulepos)
       return OGGZ_ERR_BAD_GRANULEPOS;
 
@@ -277,7 +290,7 @@ oggz_write_feed (OGGZ * oggz, ogg_packet * op, long serialno, int flush,
   /* OK -- Update stream's memory of packet details */
 
   if (!stream->metric && (oggz->flags & OGGZ_AUTO)) {
-    oggz_auto (oggz, op, serialno, NULL);
+    oggz_auto_get_granulerate (oggz, op, serialno, NULL);
   }
 
   stream->b_o_s = 0; /* The stream is henceforth no longer at bos */
@@ -321,6 +334,8 @@ oggz_write_feed (OGGZ * oggz, ogg_packet * op, long serialno, int flush,
     return -1;
   }
 
+  writer->no_more_packets = 0;
+
 #ifdef DEBUG
   printf ("oggz_write_feed: enqueued packet, queue size %d\n",
 	  oggz_vector_size (writer->packet_queue));
@@ -554,19 +569,33 @@ oggz_dequeue_packet (OGGZ * oggz, oggz_writer_packet_t ** next_zpacket)
   } else {
     *next_zpacket = oggz_vector_pop (writer->packet_queue);
 
-#ifdef DEBUG
-    printf ("oggz_dequeue_packet: dequeued packet, queue size %d\n",
-	    oggz_vector_size (writer->packet_queue));
-#endif
-
     if (*next_zpacket == NULL) {
       if (writer->hungry) {
-	ret = writer->hungry (oggz, 1, writer->hungry_user_data);
-	*next_zpacket = oggz_vector_pop (writer->packet_queue);
+        ret = writer->hungry (oggz, 1, writer->hungry_user_data);
+        *next_zpacket = oggz_vector_pop (writer->packet_queue);
+#ifdef DEBUG
+        printf ("oggz_dequeue_packet: called hungry and popped, new queue size %d\n",
+  	        oggz_vector_size (writer->packet_queue));
+#endif
+
+#ifdef DEBUG
+      } else {
+        printf ("oggz_dequeue_packet: no packet, no hungry, queue size %d\n",
+                oggz_vector_size (writer->packet_queue));
+#endif
       }
+#ifdef DEBUG
+    } else {
+    printf ("oggz_dequeue_packet: dequeued packet, queue size %d\n",
+            oggz_vector_size (writer->packet_queue));
+#endif
     }
+
   }
 
+#ifdef DEBUG
+  printf("oggz_dequeue_packeT: returning %d\n", ret);
+#endif
   return ret;
 }
 
@@ -574,7 +603,7 @@ static long
 oggz_writer_make_packet (OGGZ * oggz)
 {
   OggzWriter * writer = &oggz->x.writer;
-  oggz_writer_packet_t * zpacket, * next_zpacket;
+  oggz_writer_packet_t * zpacket, * next_zpacket = NULL;
   int cb_ret = 0;
 
 #ifdef DEBUG
@@ -618,6 +647,10 @@ oggz_writer_make_packet (OGGZ * oggz)
     }
   }
 
+#ifdef DEBUG
+  printf("oggz_writer_make_packet: cb_ret is %d\n", cb_ret);
+#endif
+  
   if (cb_ret == 0 && next_zpacket == NULL) return OGGZ_WRITE_EMPTY;
 
   return cb_ret;
@@ -626,11 +659,13 @@ oggz_writer_make_packet (OGGZ * oggz)
 long
 oggz_write_output (OGGZ * oggz, unsigned char * buf, long n)
 {
-  OggzWriter * writer = &oggz->x.writer;
+  OggzWriter * writer;
   long bytes, bytes_written = 1, remaining = n, nwritten = 0;
   int active = 1, cb_ret = 0;
 
   if (oggz == NULL) return OGGZ_ERR_BAD_OGGZ;
+ 
+  writer = &oggz->x.writer;
 
   if (!(oggz->flags & OGGZ_WRITE)) {
     return OGGZ_ERR_INVALID;
@@ -639,21 +674,54 @@ oggz_write_output (OGGZ * oggz, unsigned char * buf, long n)
   if (writer->writing) return OGGZ_ERR_RECURSIVE_WRITE;
   writer->writing = 1;
 
+#ifdef DEBUG
+  printf ("oggz_write_output: IN\n");
+#endif
+
   if ((cb_ret = oggz->cb_next) != OGGZ_CONTINUE) {
     oggz->cb_next = 0;
+    writer->writing = 0;
+    writer->no_more_packets = 0;
+    if (cb_ret == OGGZ_WRITE_EMPTY) cb_ret = 0;
     return oggz_map_return_value_to_error (cb_ret);
   }
 
   while (active && remaining > 0) {
     bytes = MIN (remaining, 1024);
 
+#ifdef DEBUG
+    printf ("oggz_write_output: write loop (%ld , %ld remain) ...\n", bytes,
+	    remaining);
+#endif
+
     while (writer->state == OGGZ_MAKING_PACKETS) {
+#ifdef DEBUG
+      	printf ("oggz_write_output: MAKING_PACKETS\n");
+#endif
       if ((cb_ret = oggz_writer_make_packet (oggz)) != OGGZ_CONTINUE) {
-	active = 0;
-	break;
+#ifdef DEBUG
+        printf ("oggz_write_output: no packets (cb_ret is %d)\n", cb_ret);
+#endif
+        if (cb_ret == OGGZ_WRITE_EMPTY) {
+          writer->flushing = 1;
+          writer->no_more_packets = 1;
+        }
+        /* At this point, in contrast to oggz_write(), we break out of this
+         * loop unconditionally.
+         */
+        active = 0;
+        break;
       }
       if (oggz_page_init (oggz)) {
-	writer->state = OGGZ_WRITING_PAGES;
+        writer->state = OGGZ_WRITING_PAGES;
+      } else {
+#ifdef DEBUG
+        printf ("oggz_write_output: unable to make page...\n");
+#endif
+        if (writer->no_more_packets) {
+          active = 0;
+          break;
+        }
       }
     }
 
@@ -661,12 +729,18 @@ oggz_write_output (OGGZ * oggz, unsigned char * buf, long n)
       bytes_written = oggz_page_copyout (oggz, buf, bytes);
 
       if (bytes_written == -1) {
-	active = 0;
+        active = 0;
         cb_ret = OGGZ_ERR_SYSTEM; /* XXX: catch next */
       } else if (bytes_written == 0) {
-	if (!oggz_page_init (oggz)) {
-	  writer->state = OGGZ_MAKING_PACKETS;
-	}
+        if (writer->no_more_packets) {
+          active = 0;
+          break;
+        } else if (!oggz_page_init (oggz)) {
+#ifdef DEBUG
+          printf ("oggz_write_output: bytes_written == 0, DONE\n");
+#endif
+          writer->state = OGGZ_MAKING_PACKETS;
+        }
       }
 
       buf += bytes_written;
@@ -676,6 +750,10 @@ oggz_write_output (OGGZ * oggz, unsigned char * buf, long n)
     }
   }
 
+#ifdef DEBUG
+  printf ("oggz_write_output: OUT %ld\n", nwritten);
+#endif
+
   writer->writing = 0;
 
   if (nwritten == 0) {
@@ -691,12 +769,14 @@ oggz_write_output (OGGZ * oggz, unsigned char * buf, long n)
 long
 oggz_write (OGGZ * oggz, long n)
 {
-  OggzWriter * writer = &oggz->x.writer;
+  OggzWriter * writer;
   long bytes, bytes_written = 1, remaining = n, nwritten = 0;
   int active = 1, cb_ret = 0;
 
   if (oggz == NULL) return OGGZ_ERR_BAD_OGGZ;
 
+  writer = &oggz->x.writer;
+
   if (!(oggz->flags & OGGZ_WRITE)) {
     return OGGZ_ERR_INVALID;
   }
@@ -710,6 +790,9 @@ oggz_write (OGGZ * oggz, long n)
 
   if ((cb_ret = oggz->cb_next) != OGGZ_CONTINUE) {
     oggz->cb_next = 0;
+    writer->writing = 0;
+    writer->no_more_packets = 0;
+    if (cb_ret == OGGZ_WRITE_EMPTY) cb_ret = 0;
     return oggz_map_return_value_to_error (cb_ret);
   }
 
@@ -722,31 +805,74 @@ oggz_write (OGGZ * oggz, long n)
 #endif
 
     while (writer->state == OGGZ_MAKING_PACKETS) {
-      if ((cb_ret = oggz_writer_make_packet (oggz)) != OGGZ_CONTINUE) {
-	active = 0;
 #ifdef DEBUG
-	printf ("oggz_write: no packets\n");
+      printf ("oggz_write: MAKING PACKETS\n");
 #endif
-	break;
+      if ((cb_ret = oggz_writer_make_packet (oggz)) != OGGZ_CONTINUE) {
+#ifdef DEBUG
+        printf ("oggz_write: no packets (cb_ret is %d)\n", cb_ret);
+#endif
+        /*
+         * if we're out of packets because we're at the end of the file,
+         * we can't finish just yet.  Instead we need to force a page flush,
+         * and write the page out.  So we set flushing and no_more_packets to
+         * 1.  This causes oggz_page_init to flush the page, then we 
+         * will switch the state to OGGZ_WRITING_PAGES, which will trigger
+         * the writing code below.
+         */
+        if (cb_ret == OGGZ_WRITE_EMPTY) {
+#ifdef DEBUG
+          printf ("oggz_write: Inferred end of data, forcing a page flush.\n");
+#endif
+          writer->flushing = 1;
+          writer->no_more_packets = 1;
+          cb_ret = OGGZ_CONTINUE;
+        } else {
+#ifdef DEBUG
+          printf ("oggz_write: Stopped by user callback.\n");
+#endif
+          active = 0;
+          break;
+        }
       }
       if (oggz_page_init (oggz)) {
-	writer->state = OGGZ_WRITING_PAGES;
+        writer->state = OGGZ_WRITING_PAGES;
+      } else {
+#ifdef DEBUG
+        printf ("oggz_write: unable to make page...\n");
+#endif
+        if (writer->no_more_packets) {
+          active = 0;
+          break;
+        }
       }
     }
 
     if (writer->state == OGGZ_WRITING_PAGES) {
       bytes_written = oggz_page_writeout (oggz, bytes);
+#ifdef DEBUG
+      printf ("oggz_write: MAKING PAGES; wrote %ld bytes\n", bytes_written);
+#endif
 
       if (bytes_written == -1) {
-	active = 0;
-	return OGGZ_ERR_SYSTEM; /* XXX: catch next */
+        active = 0;
+        return OGGZ_ERR_SYSTEM; /* XXX: catch next */
       } else if (bytes_written == 0) {
-	if (!oggz_page_init (oggz)) {
+        /*
+         * OK so we've completely written the current page.  If no_more_packets
+         * is set then that means there's no more pages after this one, so
+         * we set active to 0, break out of the loop, pack up our things and
+         * go home.
+         */
+        if (writer->no_more_packets) {
+          active = 0;
+          break;
+        } else if (!oggz_page_init (oggz)) {
 #ifdef DEBUG
-	  printf ("oggz_write: bytes_written == 0, DONE\n");
+          printf ("oggz_write: bytes_written == 0, DONE\n");
 #endif
-	  writer->state = OGGZ_MAKING_PACKETS;
-	}
+          writer->state = OGGZ_MAKING_PACKETS;
+        }
       }
 
       remaining -= bytes_written;
