@@ -84,7 +84,7 @@ read_packet (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
   static long e_o_s = 0;
 
 #ifdef DEBUG
-  printf ("%08" PRI_off_t "x: serialno %010ld, "
+  printf ("%08" PRI_OGGZ_OFF_T "x: serialno %010ld, "
 	  "granulepos %" PRId64 ", packetno %" PRId64,
 	  oggz_tell (oggz), serialno, op->granulepos, op->packetno);
 
@@ -134,7 +134,7 @@ my_io_write (void * user_handle, void * buf, size_t n)
   /* Mark that the write IO method was actually used */
   write_called++;
 
-  len = MIN (n, DATA_BUF_LEN - offset);
+  len = MIN ((int)n, DATA_BUF_LEN - offset);
   memcpy (&data_buf[offset], buf, len);
 
   offset += len;
