@@ -36,6 +36,10 @@
 #include <stdlib.h>
 #include <string.h>
 
+#ifndef WIN32
+#include <strings.h>
+#endif
+
 #include <getopt.h>
 #include <errno.h>
 
@@ -108,7 +112,7 @@ filter_page (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
 
   /* set scanning callback for keyframe calculation on theora pages only */
   if (osdata->keyframes && ogg_page_bos ((ogg_page *)og)) {
-    ident = ot_page_identify (og, NULL);
+    ident = ot_page_identify (oggz, og, NULL);
     if (ident && (strcasecmp ("theora", ident) == 0)) {
        oggz_set_read_callback (oggz, serialno, osdata->read_packet, osdata);
     }
@@ -117,14 +121,6 @@ filter_page (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
   return OGGZ_CONTINUE;
 }
 
-/* FIXME: on Mac OS X, off_t is 64-bits.  Obviously we want a nicer
- * way to do it than this, but a quick fix is a good fix */
-#ifdef __APPLE__
-#  define PRI_off_t "q"
-#else
-#  define PRI_off_t "l"
-#endif
-
 static int
 read_packet (OGGZ * oggz, ogg_packet * op, long serialno, void * user_data)
 {
