@@ -90,7 +90,7 @@ usage (char * progname)
   printf ("  -c content-type, --content-type content-type\n");
   printf ("                         Dump only the logical bitstreams for a specified\n");
   printf ("                         content-type. The following codecs are currently\n");
-  printf ("                         detected: theora, vorbis, speex, cmml\n");
+  printf ("                         detected: theora, vorbis, speex, cmml, kate\n");
   printf ("  -s serialno, --serialno serialno\n");
   printf ("                         Dump only the logical bitstream with specified serialno\n");
   printf ("  -O, --hide-offset      Hide the byte offset of each packet\n");
