@@ -53,7 +53,7 @@
 #define READ_BLOCKSIZE 1024000
 
 static void
-usage (char * progname)
+usage (const char * progname)
 {
   printf ("Usage: %s [options] filename ...\n", progname);
   printf ("Display information about one or more Ogg files and their bitstreams\n");
@@ -97,6 +97,7 @@ struct _OI_Info {
   OggzTable * tracks;
   ogg_int64_t duration;
   long length_total;
+  long overhead_length_total;
 };
 
 struct _OI_Stats {
@@ -105,10 +106,11 @@ struct _OI_Stats {
   long length_total;
   long length_min;
   long length_max;
+  long overhead_length_total;
 
   /* Pass 2 */
   long length_avg;
-  long length_deviation_total;
+  ogg_int64_t length_deviation_total;
   double length_stddev;
 };
 
@@ -151,6 +153,7 @@ oi_stats_clear (OI_Stats * stats)
   stats->length_total = 0;
   stats->length_min = LONG_MAX;
   stats->length_max = 0;
+  stats->overhead_length_total = 0;
 
   stats->length_avg = 0;
   stats->length_deviation_total = 0;
@@ -220,7 +223,7 @@ ot_fishead_print(OI_TrackInfo *oit) {
 static void
 ot_fisbone_print(OI_Info * info, OI_TrackInfo *oit) {
 
-  char *messages, *token;
+  char *allocated, *messages, *token;
   
   if (oit->has_fisbone) {
     printf("\n\tExtra information from Ogg Skeleton track:\n");
@@ -232,7 +235,7 @@ ot_fisbone_print(OI_Info * info, OI_TrackInfo *oit) {
     ot_fprint_granulepos(stdout, info->oggz, oit->fbInfo.serial_no, oit->fbInfo.start_granule);
     printf ("\n");
     printf("\tPreroll: %d\n", oit->fbInfo.preroll);
-    messages = token = _ogg_calloc(oit->fbInfo.current_header_size+1, sizeof(char));
+    allocated = messages = _ogg_calloc(oit->fbInfo.current_header_size+1, sizeof(char));
     strcpy(messages, oit->fbInfo.message_header_fields);
     printf("\tMessage Header Fields:\n");
     while (1) {
@@ -242,6 +245,7 @@ ot_fisbone_print(OI_Info * info, OI_TrackInfo *oit) {
 	break;
     }
     printf("\n");
+    _ogg_free(allocated);
   }
 }
 
@@ -254,9 +258,10 @@ oit_print (OI_Info * info, OI_TrackInfo * oit, long serialno)
   } else {
     printf ("\n???: serialno %010ld\n", serialno);
   }
-  printf ("\t%ld packets in %ld pages, %.1f packets/page\n",
+  printf ("\t%ld packets in %ld pages, %.1f packets/page, %.3f%% Ogg overhead\n",
 	  oit->packets.count, oit->pages.count,
-	  (double)oit->packets.count / (double)oit->pages.count);
+	  (double)oit->packets.count / (double)oit->pages.count,
+          oit->pages.length_stddev == 0 ? 0.0 : 100.0*oit->pages.overhead_length_total/oit->pages.length_total);
 
   if (show_length) {
     fputs("\tContent-Length: ", stdout);
@@ -313,9 +318,13 @@ oi_stats_stddev (OI_Stats * stats)
 {
   double variance;
 
-  variance = (double)stats->length_deviation_total / (double)(stats->count - 1);
-  stats->length_stddev = sqrt (variance);
-
+  if (stats->count <= 1) {
+    stats->length_stddev = 0.0;
+  }
+  else {
+    variance = (double)stats->length_deviation_total / (double)(stats->count - 1);
+    stats->length_stddev = sqrt (variance);
+  }
 }
 
 static void
@@ -346,6 +355,7 @@ read_page_pass1 (OGGZ * oggz, const ogg_page * og, long serialno, void * user_da
 
   /* Increment the total stream length */
   info->length_total += bytes;
+  info->overhead_length_total += og->header_len;
 
   /* Increment the page statistics */
   oit->pages.count++;
@@ -354,6 +364,7 @@ read_page_pass1 (OGGZ * oggz, const ogg_page * og, long serialno, void * user_da
     oit->pages.length_min = bytes;
   if (bytes > oit->pages.length_max)
     oit->pages.length_max = bytes;
+  oit->pages.overhead_length_total += og->header_len;
 
   return 0;
 }
@@ -397,8 +408,14 @@ read_packet_pass1 (OGGZ * oggz, ogg_packet * op, long serialno,
     int ret = fisbone_from_ogg(op, &fp);
     if (ret<0) return ret;
     oit = oggz_table_lookup (info->tracks, fp.serial_no);
-    oit->has_fisbone = 1;
-    oit->fbInfo = fp;
+    if (oit) {
+      oit->has_fisbone = 1;
+      oit->fbInfo = fp;
+    }
+    else {
+      fprintf(stderr, "Warning: logical stream %08x referenced by skeleton was not found\n",fp.serial_no);
+    }
+    fisbone_clear(&fp);
   } else if (!op->e_o_s && !memcmp(op->packet, FISHEAD_IDENTIFIER, 8)) {
     fishead_packet fp;
     int ret = fishead_from_ogg(op, &fp);
@@ -430,7 +447,9 @@ read_packet_pass2 (OGGZ * oggz, ogg_packet * op, long serialno,
 static int
 oi_pass1 (OGGZ * oggz, OI_Info * info)
 {
-  long n;
+  long n, serialno;
+  int ntracks, i;
+  OI_TrackInfo * oit;
 
   oggz_seek (oggz, 0, SEEK_SET);
   oggz_set_read_page (oggz, -1, read_page_pass1, info);
@@ -440,6 +459,19 @@ oi_pass1 (OGGZ * oggz, OI_Info * info)
 
   oggzinfo_apply (oit_calc_average, info);
 
+  /* Now we are at the end of the file, calculate the duration */
+  info->duration = oggz_tell_units (oggz);
+
+  /* Find the Skeleton track if present, and subtract the presentation time */
+  ntracks = oggz_table_size (info->tracks);
+  for (i = 0; i < ntracks; i++) {
+    oit = oggz_table_nth (info->tracks, i, &serialno);
+    if (oit->has_fishead) {
+      info->duration -= 1000 * oit->fhInfo.ptime_n / oit->fhInfo.ptime_d;
+      break;
+    }
+  }
+
   return 0;
 }
 
@@ -583,11 +615,10 @@ main (int argc, char ** argv)
     info.oggz = oggz;
     info.tracks = oggz_table_new ();
     info.length_total = 0;
+    info.overhead_length_total = 0;
     
     oi_pass1 (oggz, &info);
 
-    info.duration = oggz_tell_units (oggz);
-    
     oi_pass2 (oggz, &info);
     
     /* Print summary information */
